function M=create_swanspectra(S1,options)
%
% script to create a multi-modal spectrum based on
% the wave parameters Hs, Tp, DIR and DSPR
%
% Gerbrant van Vledder
% October 2015
%
% Input/ouput description
%
% S1          i  Struct with parameters of one or more spectra 
%  .hs(:)     i  Significant wave height (m)
%  .tp(:)     i  Peak period (s)
%  .mdir(:)   i  Mean wave direction (degrees)
%  .dspr(:)   i  Directional spreading (degrees)
%  .bname     i  Base name of output spectrum (including path)
% options     i  Struct with additional options
%  .fmin      i  Minimum frequency (Hz); default fmin=0.03 Hz
%  .fmax      i  Maximum frequency (Hz); default fmax=1.0  Hz
%  .delta     i  Directional resolution (); default delta=10
%  .gamma(:)  i  JONSWAP peak enhancement factor; default=gammaj(:)=3.3
%  .xloc      i  x-location of spectrum; default xloc=0
%  .yloc      i  y-location of spectrum; default yloc=0
%   
% M         o  Struct with messages
%
%  utility scripts
%  ===============
%  fjonswap        compute JONSWAP spectrum
%  z_steps         computes spacing of frequencies
%  gamma_ratio     computes ratio of 2 gamma functions
%  write_swan_sp1  write 1d spectrum to file *.sp1
%  write_swan_sp2  write 2d spectrum to file *.sp2
%
% Initialisations
%
ierr  = 0;
M.iok = 0;
rade  = 180/pi;
dera  = pi/180;
eps   = 10.^(-20);
iplot = 1;
%    

% Find structure and handles
global handles
specs=findobj('Tag','spectrumgen');

% Check input conditions
%
if nargin==1
  options = [];
end
%
% check existance of required input data
%
if isfield(S1,'hs')==1;hs=S1.hs;         else ierr=ierr+1; M.mess{ierr}='Hs not specified'; end;
if isfield(S1,'tp')==1;tp=S1.tp;         else ierr=ierr+1; M.mess{ierr}='Tp not specified'; end;
if isfield(S1,'mdir')==1;mdir=S1.mdir;   else ierr=ierr+1; M.mess{ierr}='Mdir not specified'; end;
if isfield(S1,'dspr')==1;dspr=S1.dspr;   else ierr=ierr+1; M.mess{ierr}='Dspr not specified'; end;
if isfield(S1,'bname')==1;bname=S1.bname;else ierr=ierr+1; M.mess{ierr}='Base name not specified'; end;
%
if ierr>0
  disp('Error messages from CREATE_SWANSPECTRA')
  disp('Required input parameters per spectrum:')
  disp('hs    Significant wave height (m)')
  disp('tp    Peak period (s)')
  disp('mdir  Mean wave direction ()')
  disp('dspr  Directional spreading ()')
  disp('bname Name of spectrum file')
  return
end
%
% check input data
%
sizes = [length(hs) length(tp) length(mdir) length(dspr)];
if std(sizes)>0
  ierr=ierr+1;
  disp('Input error occurred')
  M.mess{ierr} = 'Matrices with input parameters have different lengths';
  return
end
%
if min([min(hs) min(tp) min(dspr)])<=0
  ierr=ierr+1;
  disp('Input error occurred')
  M.mess{ierr} = 'Incorrect wave parameters specified';
  hs
  tp
  dspr
  return
end    
%
% set defaults for options
%
npar   = length(hs);
gammaj = ones(1,npar)*3.3;   % peak enhancement factor (-)
fmin   = 0.03;               % minimum frequency (Hz)
fmax   = 1.0;                  % maximum frequency (Hz)
ptail  = -4;                 % power of spectral tail
siga   = 0.07;               % width factor of JONSWAP spectrum
sigb   = 0.09;               % width factor of JONSWAP spectrum
delta  = 10;                 % directional resolution of 2D spectrum degrees)
xloc   = 0;                  % x-location
yloc   = 0;                  % y-location
%
% check whether options are specified
%
if isfield(options,'gamma');gammaj=options.gamma;end
if min(gammaj)<1 
  disp('Input error for optional gamma value')
  ierr = ierr+1;
  M.mess{ierr} = 'Gamma values may not be lower than 1';
  return
end
if length(gammaj)~=length(hs);
  disp('Input error for optional gamma value')
  ierr = ierr+1;
  M.mess{ierr} = 'Incorrect number of gamma values';
  return
end
%
%  update and check frequency range
%
if isfield(options,'fmin')==1; fmin=options.fmin;end
if isfield(options,'fmax')==1; fmax=options.fmax;end
if fmin<=0 || fmax<=0 || fmax<=fmin
  disp('Incorrect frequency range specified')
  disp(['fmin=' num2str(fmin) ' (Hz), fmax=' num2str(fmax) ' (Hz)'])
  ierr=ierr+1;
  M.mess{ierr} = 'Incorrect frequency range specified';
  return
end
%
if isfield(options,'delta')==1; delta=options.delta;end
if delta < 1 || delta>30
  disp('Incorrect directional spacing specified')
  disp(['delta=' num2str(delta) ' ()'])
  ierr=ierr+1;
  M.mess{ierr} = 'Incorrect directional spacing specified';
  return
end
%
if isfield(options,'xloc')==1; xloc=options.xloc;end
if isfield(options,'yloc')==1; yloc=options.yloc;end
%
ndir = 360/delta;
if abs(ndir-round(ndir))>0
  disp('Multiple of delta not equal to 360')
  disp(['delta=' num2str(delta) ' ()'])
  ierr = ierr+1;
  M.mess{ierr} = 'Incorrect directional spacing specified';
  return
end    
%    
M.iok     = 1;
%
% create 1D * 2D spectra 
%
% create frequency distribution, logarithmic spacing
% with a factor of 1.1
%
nfr  = round(log(fmax/fmin)/log(1.1)+1);
xfr  = (fmax/fmin)^(1/(nfr-1));
freq = fmin*xfr.^(0:(nfr-1))';
dirs = delta/2:delta:(360-delta/2);
%
% pre allocate memory
%
npar = length(hs);
efr  = zeros(nfr,npar);
dirf = zeros(nfr,npar);
sprf = zeros(nfr,npar);
efr2 = zeros(nfr,ndir);
%
dfr  = z_steps(freq);
%
for ipar=1:npar
  alpha = 1;
  fp    = 1./tp;
  ef0   = fjonswap(freq,alpha,fp(ipar),gammaj(ipar),siga,sigb,ptail);
  m0    = trapz(freq,ef0);
%
% add contribution of spectral tail, assuming an f-4 tail
% analytic integration up to ft=10 Hz.
%
  ft = 10;
  m0 = m0+ef0(end)*fmax^4*(fmax^(-3)-ft^(-3))/3;
%
% rescale alpha
%
  alpha = hs(ipar)/(4*sqrt(m0));
  efr(:,ipar)  = fjonswap(freq,alpha,fp(ipar),gammaj(ipar),siga,sigb,ptail);
  dirf(:,ipar) = mdir(ipar); 
  sprf(:,ipar) = dspr(ipar);
%
% compute weighted mean direction and spreading
%
  xm0  = efr(:,ipar).*dfr;
  xm1  = 1-0.5*(sprf(:,ipar)*dera).^2;
  sina = sind(dirf(:,ipar));
  cosa = cosd(dirf(:,ipar));
  a1fe(:,ipar) = xm1.*xm0.*cosa;
  b1fe(:,ipar) = xm1.*xm0.*sina;
%
% add to 2D spectrum
%
  sigma = max(0,sprf(:,ipar)*dera);
  s     = 2./sigma.^2;
  a     = gamma_ratio(s)/(2*sqrt(pi));
%
  for ifr=1:nfr
    alfa  = 180-abs(180-abs(dirs-dirf(ifr,ipar)));
    ee    = a(ifr)*cosd(alfa/2).^(2*s(ifr));
    efr2(ifr,:) = efr2(ifr,:) + efr(ifr,ipar).*ee;
  end  
end  
%
% sum weighted Fourier coefficients per frequency
%
a1f = sum(a1fe,2);
b1f = sum(b1fe,2);
%
% compute mean direction and spreading per frequency
%
dirm  = atan2(b1f,a1f)*rade;;
m0f   = sum(efr,2).*dfr
m1f   = sqrt(a1f.^2+b1f.^2)./m0f;
dspr  = sqrt(2*(1-m1f))*rade;
%
SW1.dirm = dirm;
SW1.dspr = dspr;
SW1.freq = freq;
%
efr(efr<eps) = eps;
SW1.efr        = sum(efr,2);
%
% write information to struct
%
SW1.xloc = xloc;
SW1.yloc = yloc;
%
options1d.fspec = bname;
write_swan_sp1(SW1,options1d);
%
clear S2
SW2.coortype = 'LOCATIONS'
SW2.xloc     = xloc;
SW2.yloc     = yloc;
SW2.dirs     = dirs;
SW2.freq     = freq;
SW2.factor   = max(max(efr2))/999999
SW2.spec2d   = efr2;
option2.fspec  = bname;
write_swan_sp2(SW2,option2)
%
if iplot==1

try
    if isfield(handles.specgen,'fig2')==1
        close(handles.specgen.fig2)
    end
catch
end

% get locations of tickmarks

  if (fmax-fmin)<0.6
    df = 0.05;
  else
    df = 0.2;
  end
  xlines = [df:df:fmax];  

% create new figure
handles.specgen.fig2=figure('Name','SpecGen - Spectra','NumberTitle','off','position',[100 100 700 400])
  
% plot 1D spectrum
  subplot(1,2,1)
  plot(freq,SW1.efr,'k-')
  grid on
  xlabel('f (Hz)')
  ylabel('E(f) (m^2/Hz)')
  set(gca,'xlim',[0 0.5],'box','on')
  set(gca,'xtick',xlines)
  title('SWAN 1D Spectrum','fontsize',12)
  
% plot 2d spectrum
  subplot(1,2,2)
  optie.ttext=('SWAN 2D Spectrum')
  optie.ffmax=0.5
  isopol3(freq,dirs,efr2,optie)
  axis('equal')
  
  % Set handles to 'Userdata' property
set(specs,'UserData',handles)
  
end 