function write_swan_sp1(S,options)
%
% write swan sp1 spectrum to file 
% Gerbrant van Vledder
% October 2015
%
% Required input
%
% S.freq(:)      Frequencies (Hz)
% S.efr(:,:)     Spectral densities (m^2/Hz)
% S.xloc(:)      x-locations
% S.yloc(:)      y-locations
%
% check input
%
if isfield(S,'freq')==0;error('WRITE_SWAN_SP1: FREQ not specified');end
if isfield(S,'efr')==0;error('WRITE_SWAN_SP1: EFR not specified');end
if isfield(S,'xloc')==0;error('WRITE_SWAN_SP1: XLOC not specified');end
if isfield(S,'yloc')==0;error('WRITE_SWAN_SP1: YLOC not specified');end
%
% set defaults
%
fspec = 'dummy.sp1';       % dummy file name
iproj = 0;                 % switch to write comment line on project
ktime = 0;                 % switch to write time dependent data
ntime = 1;                 % at least one set of spectra is written
quant = {'VaDens' 'NDIR' 'DSPRDEGR'};  % quantities
unit  = {'m2/Hz' 'degr' 'degr'};       % units
exc   = [-99 -999 -9];                 % exception values
eps   = 10^(-20);                      % very small number
coortype = 'LOCATIONS'; 
frtype   = 'AFREQ';
%
if nargin==1
  options.fspec = fspec;
end
%
if isfield(options,'fspec')==1;   fspec    = options.fspec ;end;
if isfield(options,'project')==1; project  = options.project;iproj=1;end
if isfield(S,'t')==1; ktime=1;    ntime    = length(S.t); end
if isfield(S,'coortype')==1;      coortype = S.coortype;end
if isfield(S,'frtype')==1;        frtype   = S.frtype;end
if isfield(S,'quant')==1;         quant    = S.quant;end
if isfield(S,'unit')==1;          unit     = S.unit;end
if isfield(S,'exc')==1;           exc      = S.exc;end
nquant = length(quant);
nfreq  = length(S.freq);
%
% check extension of fspec
%
idot = strfind(fspec,'.');
if isempty(idot)
  fspec = [fspec '.sp1'];
end
%
% open new file
%
fid = fopen(fspec,'w');
fprintf(fid,'%s\n','SWAN 1     SWAN standard spectra file, version');
fprintf(fid,'%s\n','$  File written by script write_swan_sp1, version 1');
if iproj==1
  fprintf(fid,'%s\n',['$  Project: ' project]);
end
%
% switch to write time header
%
if ktime==1
  fprintf(fid,'%s\n',['TIME          time-dependent data']);    
  fprintf(fid,'%s\n',['      1       time coding options']);    
end    
% 
% location data
%
fprintf(fid,'%s\n',coortype);
nloc = length(S.xloc);
fprintf(fid,'%6i\n',nloc);
%
if strcmp(coortype,'LONLAT')==1
  for iloc=1:nloc
    fprintf(fid,'%12.6f %12.6f\n',[S.xloc(iloc) S.yloc(iloc)]);
  end
elseif strcmp(coortype,'LOCATIONS')==1  
  for iloc=1:nloc
    fprintf(fid,'%12.4f %12.4f\n',[S.xloc(iloc) S.yloc(iloc)]);
  end
end
%
% frequencies
%
fprintf(fid,'%s\n',frtype);
% nfreq = length(S.freq);
fprintf(fid,'%6i\n',nfreq);
for ifr=1:nfreq
  fprintf(fid,'%10.4f\n',S.freq(ifr));  
end
%
% general information, not yet from struct
%
fprintf(fid,'%s\n','QUANT');
fprintf(fid,'%6i\n',nquant); %  number of quantities in table');
for iquant=1:nquant
   fprintf(fid,'%s\n',quant{iquant});
   fprintf(fid,'%s\n',unit{iquant});
   fprintf(fid,'%8i\n',exc(iquant));
end
% fprintf(fid,'%s\n','VaDens      variance densities in m2/Hz');
% fprintf(fid,'%s\n','m2/Hz       unit');
% fprintf(fid,'%s\n','  -99       exception value');
% fprintf(fid,'%s\n','NDIR        average nautical direction in degr');
% fprintf(fid,'%s\n','degr        unit');
% fprintf(fid,'%s\n',' -999       exception value');
% fprintf(fid,'%s\n','DSPRDEGR    directional spreading');
% fprintf(fid,'%s\n','degr        unit');
% fprintf(fid,'%s\n','   -9       exception value');
% %
% loop over times and loop over output locations
%
if ktime==1
  wb=waitbar(0,'Writing spectra for all times');
end  
%
for jtime=1:ntime
  if ktime==0
    wb=waitbar(0,'Writing spectra for all locations');
    efr  = S.efr;
    dirm = S.dirm;
    dspr = S.dspr;
  else  
    ctime = [datestr(S.t(jtime),'yyyymmdd.HHMMSS')  '                date and time'];
    fprintf(fid,'%s\n',ctime);
    waitbar(jtime/ntime,wb);
    efr  = squeeze(S.efr(jtime,:,:));
    dirm = squeeze(S.dirm(jtime,:,:));
    dspr = squeeze(S.dspr(jtime,:,:));
  end
%
% set small energy densities to dummies as well as direction and spreading
%
%   [efr dirm dspr]
%   ismall = find(efr<eps);
%   efr(ismall)  = exc(1);
%   dirm(ismall) = exc(2);
%   dspr(ismall) = exc(3);
%   [efr dirm dspr]
%  
  for iloc=1:nloc
    if ktime==0
      waitbar(iloc/nloc,wb);  
    end
    cloc = ['LOCATION ' num2str(iloc,'%5i')]; 
    if sum(efr(:,iloc))<eps
      fprintf(fid,'%s\n','NODATA');        
    else
      fprintf(fid,'%s\n',cloc);
      for ifreq=1:nfreq
         data = [efr(ifreq,iloc) dirm(ifreq,iloc) dspr(ifreq,iloc)];
         if sum(isnan(data))>0 || efr(ifreq,iloc) < eps
           data = [-99 -999 -9];  
         end
         fprintf(fid,'%10.4E %6.1f %5.1f',data);
         fprintf(fid,'%s\n',' ');      
      end 
    end    
  end
end
%
close(wb)
fclose(fid);

